@@ -13,9 +13,20 @@ from typing import List, Optional
 import networkx as nx
 import numpy as np
 import torch
+from loguru import logger as log
 
 from graphein.utils.utils import import_message
 
+try:
+    import torch
+except ImportError:
+    import_message(
+        submodule="graphein.ml.conversion",
+        package="torch",
+        pip_install=True,
+        conda_channel="pytorch",
+    )
+
 try:
     import torch_geometric
     from torch_geometric.data import Data
@@ -121,7 +132,6 @@ class GraphFormatConvertor:
                 columns = [
                     "edge_index",
                     "coords",
-                    "dist_mat",
                     "name",
                     "node_id",
                 ]
@@ -130,7 +140,6 @@ class GraphFormatConvertor:
                     "b_factor",
                     "chain_id",
                     "coords",
-                    "dist_mat",
                     "edge_index",
                     "kind",
                     "name",
@@ -264,31 +273,56 @@ class GraphFormatConvertor:
         G = nx.convert_node_labels_to_integers(G)
 
         # Construct Edge Index
-        edge_index = torch.LongTensor(list(G.edges)).t().contiguous()
+        edge_index = (
+            torch.LongTensor(list(G.edges)).t().contiguous().view(2, -1)
+        )
 
         # Add node features
+        node_feature_names = G.nodes(data=True)[0].keys()
         for i, (_, feat_dict) in enumerate(G.nodes(data=True)):
             for key, value in feat_dict.items():
-                if str(key) in self.columns:
-                    data[str(key)] = (
-                        [value] if i == 0 else data[str(key)] + [value]
-                    )
+                key = str(key)
+                if key in self.columns:
+                    if i == 0:
+                        data[key] = []
+                    data[key].append(value)
 
         # Add edge features
         for i, (_, _, feat_dict) in enumerate(G.edges(data=True)):
             for key, value in feat_dict.items():
-                if str(key) in self.columns:
-                    data[str(key)] = (
-                        list(value) if i == 0 else data[str(key)] + list(value)
-                    )
+                key = str(key)
+                if key in self.columns or key == "kind":
+                    if i == 0:
+                        data[key] = []
+                    data[key].append(value)
 
         # Add graph-level features
         for feat_name in G.graph:
             if str(feat_name) in self.columns:
-                data[str(feat_name)] = [G.graph[feat_name]]
-
+                if str(feat_name) not in node_feature_names:
+                    data[str(feat_name)] = G.graph[feat_name]
         if "edge_index" in self.columns:
-            data["edge_index"] = edge_index.view(2, -1)
+            data["edge_index"] = edge_index
+
+        # Split edge index by edge kind
+        kind_strs = np.array(
+            list(map(lambda x: "_".join(x), data.get("kind", [])))
+        )
+        for kind in set(kind_strs):
+            key = f"edge_index_{kind}"
+            if key in self.columns:
+                mask = kind_strs == kind
+                data[key] = edge_index[:, mask]
+        if "kind" not in self.columns and data.get("kind"):
+            del data["kind"]
+
+        # Convert everything possible to torch.Tensors
+        for key, val in data.items():
+            try:
+                data[key] = torch.tensor(np.array(val))
+            except Exception as e:
+                log.warning(e)
+                pass
 
         data = Data.from_dict(data)
         data.num_nodes = G.number_of_nodes()
