@@ -6,12 +6,12 @@
 # Code Repository: https://github.com/a-r-j/graphein
 from __future__ import annotations
 
-import logging as log
 import os
 from pathlib import Path
 from typing import Callable, Dict, Generator, List, Optional
 
 import networkx as nx
+from loguru import logger as log
 from tqdm import tqdm
 
 from graphein.ml.conversion import GraphFormatConvertor
@@ -73,7 +73,8 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
         :type root: str
         :param name: Name of the dataset. Will be saved to ``data_$name.pt``.
         :type name: str
-        :param pdb_paths: List of full path of pdb files to load. Defaults to ``None``.
+        :param pdb_paths: List of full path of pdb files to load. Defaults to
+            ``None``.
         :type pdb_paths: Optional[List[str]], optional
         :param pdb_codes: List of PDB codes to download and parse from the PDB.
             Defaults to None.
@@ -146,7 +147,8 @@ class InMemoryProteinGraphDataset(InMemoryDataset):
                 self.structures = pdb_codes
             elif self.uniprot_ids:
                 self.structures = uniprot_ids
-        # Use local saved pdb_files instead of download or move them to self.root/raw dir
+        # Use local saved pdb_files instead of download or move them to
+        # self.root/raw dir
         else:
             if isinstance(self.pdb_paths, list):
                 self.structures = [
@@ -356,7 +358,8 @@ class ProteinGraphDataset(Dataset):
 
         :param root: Root directory where the dataset should be saved.
         :type root: str
-        :param pdb_paths: List of full path of pdb files to load. Defaults to ``None``.
+        :param pdb_paths: List of full path of pdb files to load. Defaults to
+            ``None``.
         :type pdb_paths: Optional[List[str]], optional
         :param pdb_codes: List of PDB codes to download and parse from the PDB.
             Defaults to ``None``.
@@ -427,7 +430,8 @@ class ProteinGraphDataset(Dataset):
                 self.structures = pdb_codes
             elif self.uniprot_ids:
                 self.structures = uniprot_ids
-        # Use local saved pdb_files instead of download or move them to self.root/raw dir
+        # Use local saved pdb_files instead of download or move them to
+        # self.root/raw dir
         else:
             if isinstance(self.pdb_paths, list):
                 self.structures = [
@@ -658,7 +662,8 @@ class ProteinGraphDataset(Dataset):
             return torch.load(
                 os.path.join(
                     self.processed_dir,
-                    f"{self.structures[idx]}_{self.chain_selection_map[idx]}.pt",
+                    f"{self.structures[idx]}_\
+                        {self.chain_selection_map[idx]}.pt",
                 )
             )
         else:
