@@ -1,6 +1,8 @@
 from .atoms import *
+from .chembl import *
 from .config import *
 from .edges import *
 from .features import *
 from .graphs import *
 from .visualisation import *
+from .zinc import *
