@@ -1,13 +1,10 @@
 """Provides utility functions for use across Graphein."""
-from __future__ import annotations
-
-import logging
-
 # Graphein
 # Author: Arian Jamasb <arian@jamasb.io>
 # License: MIT
 # Project Website: https://github.com/a-r-j/graphein
 # Code Repository: https://github.com/a-r-j/graphein
+
 import os
 import tempfile
 from functools import lru_cache, partial
@@ -19,14 +16,14 @@ from urllib.request import urlopen
 
 import networkx as nx
 import pandas as pd
+import requests
 import wget
 from biopandas.pdb import PandasPdb
+from loguru import logger as log
 from tqdm.contrib.concurrent import process_map
 
 from .resi_atoms import BACKBONE_ATOMS, RESI_THREE_TO_1
 
-log = logging.getLogger(__name__)
-
 
 class ProteinGraphConfigurationError(Exception):
     """
@@ -60,9 +57,40 @@ def get_obsolete_mapping() -> Dict[str, str]:
     return obs_dict
 
 
+def read_fasta(file_path: str) -> Dict[str, str]:
+    """
+    Reads a FASTA file and returns a dictionary mapping sequence names to
+    their identifiers.
+
+    :param file_path: Path to FASTA file.
+    :type file_path: str
+    :return: Dictionary mapping sequence names to their identifiers.
+    :rtype: Dict[str, str]
+    """
+    sequences = {}
+    current_sequence_name = None
+    current_sequence = ""
+
+    with open(file_path, "r") as file:
+        for line in file:
+            if line.startswith(">"):
+                if current_sequence_name:
+                    sequences[current_sequence_name] = current_sequence
+
+                current_sequence_name = line[1:].strip()
+                current_sequence = ""
+            else:
+                current_sequence += line.strip()
+
+        if current_sequence_name:
+            sequences[current_sequence_name] = current_sequence
+
+    return sequences
+
+
 def download_pdb_multiprocessing(
     pdb_codes: List[str],
-    out_dir: Union[str, Path],
+    out_dir: Union[str, Path],  # type: ignore
     overwrite: bool = False,
     strict: bool = False,
     max_workers: int = 16,
@@ -146,7 +174,8 @@ def download_pdb(
             return download_pdb(new_pdb, out_dir, overwrite=overwrite)
         except KeyError:
             log.warning(
-                f"PDB {pdb_code} not found. Possibly too large; large structures are only provided as mmCIF files."
+                f"PDB {pdb_code} not found. Possibly too large; large \
+                    structures are only provided as mmCIF files."
             )
             return
 
@@ -162,7 +191,7 @@ def download_pdb(
             out=str(out_dir / f"{pdb_code}.pdb"),
         )
     except HTTPError:
-        log.info(f"PDB {pdb_code} not found.")
+        log.warning(f"PDB {pdb_code} not found.")
 
     # Check file exists
     if strict:
@@ -238,9 +267,10 @@ def download_alphafold_structure(
     pdb: bool = True,
     mmcif: bool = False,
     aligned_score: bool = True,
-) -> Union[str, Tuple[str, str]]:
+) -> Union[str, Tuple[str, str], None]:
     """
-    Downloads a structure from the Alphafold EBI database (https://alphafold.ebi.ac.uk/files/").
+    Downloads a structure from the Alphafold EBI database
+    (https://alphafold.ebi.ac.uk/files/").
 
     :param uniprot_id: UniProt ID of desired protein.
     :type uniprot_id: str
@@ -272,7 +302,14 @@ def download_alphafold_structure(
         query_url = f"{BASE_URL}AF-{uniprot_id}-F1-model_v{version}.cif"
     if pdb:
         query_url = f"{BASE_URL}AF-{uniprot_id}-F1-model_v{version}.pdb"
-    structure_filename = wget.download(query_url, out=out_dir)
+
+    try:
+        structure_filename = wget.download(query_url, out=out_dir)
+    except HTTPError:
+        log.warning(
+            f"No structure found for {uniprot_id}. Used URL: {query_url}"
+        )
+        return None
 
     if rename:
         extension = ".pdb" if pdb else ".cif"
@@ -418,7 +455,8 @@ def save_rgroup_df_to_pdb(
 def is_tool(name: str) -> bool:
     """Checks whether ``name`` is on ``PATH`` and is marked as an executable.
 
-    Source: https://stackoverflow.com/questions/11210104/check-if-a-program-exists-from-a-python-script
+    Source:
+    https://stackoverflow.com/questions/11210104/check-if-a-program-exists-from-a-python-script
 
     :param name: Name of program to check for execution ability.
     :type name: str
@@ -426,3 +464,43 @@ def is_tool(name: str) -> bool:
     :rtype: bool
     """
     return which(name) is not None
+
+
+def esmfold(
+    sequence: str,
+    out_path: Optional[str] = None,
+    version: int = 1,
+    format: str = "pdb",
+):
+    """Fold a protein sequence using the ESMFold model from the ESMFold server at
+    https://api.esmatlas.com/foldSequence/v1/pdb/.
+
+
+    Parameters
+    ----------
+    sequence : str
+        A protein sequence in one-letter code.
+    out_path : str, optional
+        Path to save the PDB file to. If `None`, the file is not saved.
+        Defaults to `None`.
+    version : int, optional
+        The version of the ESMFold model to use. Defaults to `1`.
+    Returns
+    --------
+    self
+    """
+    URL = f"https://api.esmatlas.com/foldSequence/v{version}/{format}/"
+
+    headers: Dict[str, str] = {
+        "Content-Type": "application/x-www-form-urlencoded",
+    }
+
+    cif = requests.post(URL, data=sequence, headers=headers).text
+    # append header
+    header = "\n".join(
+        [f"data_{sequence}", "#", f"_entry.id\t{sequence}", "#\n"]
+    )
+    cif = header + cif
+    if out_path is not None:
+        with open(out_path, "w") as f:
+            f.write(cif)
