@@ -5,11 +5,11 @@
 # Project Website: https://github.com/a-r-j/graphein
 # Code Repository: https://github.com/a-r-j/graphein
 
-import logging
 from typing import List, Optional, Tuple, Union
 
 import networkx as nx
 import numpy as np
+from loguru import logger as log
 
 import graphein.protein.subgraphs as protein
 from graphein.rna.constants import (
@@ -18,8 +18,6 @@ from graphein.rna.constants import (
     RNA_BACKBONE_ATOMS,
 )
 
-log = logging.getLogger(__name__)
-
 
 def extract_subgraph_from_node_list(
     g,
@@ -36,11 +34,14 @@ def extract_subgraph_from_node_list(
     :type g: nx.Graph
     :param node_list: The list of nodes to extract.
     :type node_list: List[str], optional
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` DataFrame of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -68,15 +69,19 @@ def extract_subgraph_from_backbone(
 ):
     """Extracts a subgraph from an RNA structure retaining only backbone atoms.
 
-    Backbone atoms are defined in :ref:`graphein.rna.constants.RNA_BACKBONE_ATOMS`.
+    Backbone atoms are defined in
+    :ref:`graphein.rna.constants.RNA_BACKBONE_ATOMS`.
 
     :param g: RNA Structure Graph to extract subgraph from.
     :type g: nx.Graph
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -104,15 +109,20 @@ def extract_subgraph_from_bases(
 ):
     """Extracts a subgraph from an RNA structure retaining only base atoms.
 
-    Backbone atoms are defined in :ref:`graphein.rna.constancts.RNA_BACKBONE_ATOMS`. We exclude these to perform the selection.
+    Backbone atoms are defined in
+    :ref:`graphein.rna.constancts.RNA_BACKBONE_ATOMS`. We exclude these to
+    perform the selection.
 
     :param g: RNA Structure Graph to extract subgraph from.
     :type g: nx.Graph
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -141,15 +151,19 @@ def extract_subgraph_from_phosphoric_acid(
 ):
     """Extracts a subgraph from an RNA structure retaining only base atoms.
 
-    Phosphoric acid atoms are defined in :ref:`graphein.rna.constancts.PHOSPHORIC_ACID_ATOMS`.
+    Phosphoric acid atoms are defined in
+    :ref:`graphein.rna.constancts.PHOSPHORIC_ACID_ATOMS`.
 
     :param g: RNA Structure Graph to extract subgraph from.
     :type g: nx.Graph
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -177,15 +191,18 @@ def extract_subgraph_from_ribose(
 ):
     """Extracts a subgraph from an RNA structure retaining only base atoms.
 
-    Ribose atoms are defined in :ref:`graphein.rna.constancts.RIBOSE_ATOMS`.
+    Ribose atoms are defined in :ref:`graphein.rna.constants.RIBOSE_ATOMS`.
 
     :param g: RNA Structure Graph to extract subgraph from.
     :type g: nx.Graph
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -221,11 +238,14 @@ def extract_subgraph_from_point(
     :type centre_point: Tuple[float, float, float]
     :param radius: The radius of the subgraph.
     :type radius: float
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+    graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -259,11 +279,14 @@ def extract_subgraph_from_atom_types(
     :type g: nx.Graph
     :param atom_types: The list of atom types to extract.
     :type atom_types: List[str]
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -290,17 +313,22 @@ def extract_subgraph_from_residue_types(
     inverse: bool = False,
     return_node_list: bool = False,
 ) -> Union[nx.Graph, List[str]]:
-    """Extracts a subgraph from a graph based on a list of allowable (nucleotide) residue types.
+    """Extracts a subgraph from a graph based on a list of allowable
+    (nucleotide) residue types.
 
     :param g: The graph to extract the subgraph from.
     :type g: nx.Graph
-    :param residue_types: List of allowable residue types (1-letter residue names). E.g. ``["A", "G"]``
+    :param residue_types: List of allowable residue types (1-letter residue
+        names). E.g. ``["A", "G"]``.
     :type residue_types: List[str]
-    :param filter_dataframe: Whether to filer the ``pdb_df`` of the graph, defaults to ``True``
+    :param filter_dataframe: Whether to filer the ``pdb_df`` of the graph,
+        defaults to ``True``.
     :type filter_dataframe: bool, optional
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -331,13 +359,16 @@ def extract_subgraph_from_chains(
 
     :param g: The graph to extract the subgraph from.
     :type g: nx.Graph
-    :param chains: The chain(s) to extract. E.g. ``["A", "C"]``
+    :param chains: The chain(s) to extract. E.g. ``["A", "C"]``.
     :type chains: List[str]
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -368,13 +399,16 @@ def extract_subgraph_by_sequence_position(
 
     :param g: The graph to extract the subgraph from.
     :type g: nx.Graph
-    :param chain: The sequence positions to extract. E.g. ``[1, 3, 5, 7]``
+    :param chain: The sequence positions to extract. E.g. ``[1, 3, 5, 7]``.
     :type chain: List[int]
-    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the graph. Defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` dataframe of the
+        graph. Defaults to ``True``.
     :type filter_dataframe: bool
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection. Defaults to ``False``.
     :type inverse: bool
@@ -407,15 +441,19 @@ def extract_subgraph_by_bond_type(
     :type g: nx.Graph
     :param bond_types: List of allowable bond types.
     :type bond_types: List[str]
-    :param filter_dataframe: Whether to filter the ``pdb_df`` of the graph, defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` of the graph,
+        defaults to ``True``.
     :type filter_dataframe: bool, optional
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection, defaults to ``False``.
     :type inverse: bool, optional
-    :param return_node_list: Whether to return the node list, defaults to ``False``.
+    :param return_node_list: Whether to return the node list, defaults to
+        ``False``.
     :type return_node_list: bool, optional
     :return: The subgraph or node list if ``return_node_list=True``.
     :rtype: Union[nx.Graph, List[str]]
@@ -450,17 +488,22 @@ def extract_k_hop_subgraph(
     :type central_node: str
     :param k: The number of hops to extract.
     :type k: int
-    :param k_only: Whether to only extract the exact k-hop subgraph (e.g. include 2-hop neighbours in 5-hop graph). Defaults to ``False``.
+    :param k_only: Whether to only extract the exact k-hop subgraph (e.g.
+        include 2-hop neighbours in 5-hop graph). Defaults to ``False``.
     :type k_only: bool
-    :param filter_dataframe: Whether to filter the ``pdb_df`` of the graph, defaults to ``True``.
+    :param filter_dataframe: Whether to filter the ``pdb_df`` of the graph,
+        defaults to ``True``.
     :type filter_dataframe: bool, optional
-    :param update_coords: Whether to update the coordinates of the graph. Defaults to ``True``.
+    :param update_coords: Whether to update the coordinates of the graph.
+        Defaults to ``True``.
     :type update_coords: bool
-    :param recompute_distmat: Whether to recompute the distance matrix of the graph. Defaults to ``False``.
+    :param recompute_distmat: Whether to recompute the distance matrix of the
+        graph. Defaults to ``False``.
     :type recompute_distmat: bool
     :param inverse: Whether to inverse the selection, defaults to ``False``.
     :type inverse: bool, optional
-    :param return_node_list: Whether to return the node list. Defaults to ``False``.
+    :param return_node_list: Whether to return the node list. Defaults to
+        ``False``.
     :type return_node_list: bool
     :return: The subgraph or node list if ``return_node_list=True``.
     :rtype: Union[nx.Graph, List[str]]
